

  
/***
*
*	Copyright (c) 1996-2001, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   This source code contains proprietary and confidential information of
*   Valve LLC and its suppliers.  Access to this code is restricted to
*   persons who have executed a written SDK license with Valve.  Any access,
*   use or distribution of this code by or to any unlicensed person is illegal.
*
****/
//=========================================================
// Zombie
//=========================================================

// UNDONE: Don't flinch every time you get hit

#include	"extdll.h"
#include	"util.h"
#include	"cbase.h"
#include	"monsters.h"
#include	"schedule.h"

#include        "weapons.h"


#include        "effects.h"


//=========================================================
// Monster's Anim Events Go Here
//=========================================================
#define	ZOMBIE_AE_ATTACK_RIGHT		0x01
#define	ZOMBIE_AE_ATTACK_LEFT		0x02
#define	ZOMBIE_AE_ATTACK_BOTH		0x03

#define ZOMBIE_FLINCH_DELAY			2		// at most one flinch every n secs

class CZombie : public CBaseMonster
{
public:
	void Spawn( void );
	void Precache( void );
	void SetYawSpeed( void );
	int  Classify ( void );
	void HandleAnimEvent( MonsterEvent_t *pEvent );
	int IgnoreConditions ( void );

	float m_flNextFlinch;

        //void PainSound( void );
        //void AlertSound( void );
        //void IdleSound( void );
        //void AttackSound( void );

        //static const char *pAttackSounds[];
        //static const char *pIdleSounds[];
        //static const char *pAlertSounds[];
        //static const char *pPainSounds[];
        //static const char *pAttackHitSounds[];
        //static const char *pAttackMissSounds[];

	// No range attacks
	BOOL CheckRangeAttack1 ( float flDot, float flDist ) { return FALSE; }
	BOOL CheckRangeAttack2 ( float flDot, float flDist ) { return FALSE; }
	int TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType );

        //bc custom additions
        EHANDLE m_hOwner;
        int IRelationship ( CBaseEntity *pTarget );
        void Killed( entvars_t *pevAttacker, int iGib );
        void PrescheduleThink( void );
        //int m_iTrail;

private:
        CSprite *Outline;
        CSprite *Halo;
};

LINK_ENTITY_TO_CLASS( monster_zombie, CZombie );

//=========================================================
// Classify - indicates this monster's place in the 
// relationship table.
//=========================================================
int	CZombie :: Classify ( void )
{
//        return  CLASS_ALIEN_MONSTER;
        return  CLASS_PLAYER_ALLY;
}

//=========================================================
// SetYawSpeed - allows each sequence to have a different
// turn rate associated with it.
//=========================================================
void CZombie :: SetYawSpeed ( void )
{
	int ys;

        ys = 256;//120 wow fast!

#if 0
	switch ( m_Activity )
	{
	}
#endif

	pev->yaw_speed = ys;
}



//bc
int CZombie :: IRelationship ( CBaseEntity *pTarget )
{
		return R_NO;
}




int CZombie :: TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType )
{
	return CBaseMonster::TakeDamage( pevInflictor, pevAttacker, flDamage, bitsDamageType );
}

//=========================================================
// HandleAnimEvent - catches the monster-specific messages
// that occur when tagged animation frames are played.
//=========================================================
void CZombie :: HandleAnimEvent( MonsterEvent_t *pEvent )
{
        CBaseMonster::HandleAnimEvent( pEvent );
}

//=========================================================
// Spawn
//=========================================================
void CZombie :: Spawn()
{
	Precache( );

        SET_MODEL(ENT(pev), "models/unit.mdl");
	UTIL_SetSize( pev, VEC_HUMAN_HULL_MIN, VEC_HUMAN_HULL_MAX );

        pev->solid              = SOLID_SLIDEBOX;
	pev->movetype		= MOVETYPE_STEP;
        m_bloodColor            = BLOOD_COLOR_RED;
        pev->health             = gSkillData.zombieHealth;
	pev->view_ofs		= VEC_VIEW;// position of the eyes relative to monster's origin.
	m_flFieldOfView		= 0.5;// indicates the width of this monster's forward view cone ( as a dotproduct result )
	m_MonsterState		= MONSTERSTATE_NONE;
	m_afCapability		= bits_CAP_DOORS_GROUP;

//BC
        if ( pev->owner )
                m_hOwner = Instance( pev->owner );


        IsActive = 0;


//bbc we are spawned with an "active.mdl" model attachment.
        Halo = CSprite::SpriteCreate( "models/active.mdl", pev->origin, FALSE );
        if (Halo)
        {
                Halo->SetAttachment( edict(), 2 );
                Halo->pev->effects |= EF_NODRAW; //starts invisible
                Halo->pev->rendermode = kRenderTransAdd;
                Halo->pev->renderamt = 255;
        }


        // (?????????)
        pev->rendermode = kRenderTransAlpha;
        pev->renderamt = 255;
        
        //choose a random goofy faceplate!
        pev->skin = RANDOM_LONG(0,19);

        //bcc
        //this is called only once. teamcolor identifier.
        Outline = CSprite::SpriteCreate( "models/select.mdl", pev->origin, FALSE );
        if (Outline)
        {
                Outline->SetAttachment( edict(), 0 );
                //bc
                //CBaseEntity *pPlayer = (CBaseEntity *)pev->owner;

                if (pev->owner)
                {
                        CBaseEntity *pPlayer = m_hOwner;
                        const char *pTeamName = pPlayer->TeamID();

                        if (FStrEq(pTeamName, "Blue"))
                        {
                                if (pev->body != 0)
                                        pev->body = 0;
                                if (Outline->pev->skin != 0)
                                        Outline->pev->skin = 0;
                        }
                        else
                        {
                                if (pev->body != 1)
                                        pev->body = 1;
                                if (Outline->pev->skin != 1)
                                        Outline->pev->skin = 1;
                        }
                }

                Outline->pev->rendermode = kRenderTransAdd;
                Outline->pev->renderamt = 255;
        }

	MonsterInit();
}

//=========================================================
// Precache - precaches all resources this monster needs
//=========================================================
void CZombie :: Precache()
{
        PRECACHE_MODEL("models/unit.mdl");  //the dude.
        //PRECACHE_MODEL("sprites/i_skull.spr");
        PRECACHE_MODEL("models/flowers.mdl");
        //m_iTrail = PRECACHE_MODEL("sprites/smoke.spr");
}




void CZombie :: PrescheduleThink( void )
{
//bc
        if (IsActive)//unit is currently active.
        {
                if (Halo->pev->effects & EF_NODRAW)//if halo is invisible
                {
                        Halo->pev->effects &= ~EF_NODRAW;//make halo visible.
                }
        }
        else if (!(Halo->pev->effects & EF_NODRAW))//if halo is visible + unit inactive
        {
                Halo->pev->effects |= EF_NODRAW;//make halo invisible.
        }
}









void CZombie::Killed( entvars_t *pevAttacker, int iGib )
{
        //remove the unit outline.
        if (Outline)
                UTIL_Remove( Outline );

        if (IsActive)
                IsActive = FALSE;


        CSprite *m_pBall = CSprite::SpriteCreate( "models/flowers.mdl", pev->origin, FALSE );
        m_pBall->pev->angles.y = RANDOM_LONG(1,359);
        if (FStrEq(m_hOwner->TeamID(), "Red"))
                m_pBall->pev->skin = 1;


        CBaseMonster::Killed( pevAttacker, iGib );
        
}


//=========================================================
// AI Schedules Specific to this monster
//=========================================================



int CZombie::IgnoreConditions ( void )
{
	int iIgnore = CBaseMonster::IgnoreConditions();

	if ((m_Activity == ACT_MELEE_ATTACK1) || (m_Activity == ACT_MELEE_ATTACK1))
	{
#if 0
		if (pev->health < 20)
			iIgnore |= (bits_COND_LIGHT_DAMAGE|bits_COND_HEAVY_DAMAGE);
		else
#endif			
		if (m_flNextFlinch >= gpGlobals->time)
			iIgnore |= (bits_COND_LIGHT_DAMAGE|bits_COND_HEAVY_DAMAGE);
	}

	if ((m_Activity == ACT_SMALL_FLINCH) || (m_Activity == ACT_BIG_FLINCH))
	{
		if (m_flNextFlinch < gpGlobals->time)
			m_flNextFlinch = gpGlobals->time + ZOMBIE_FLINCH_DELAY;
	}

	return iIgnore;
	
}
